function init()
    m.indexButtonGo = 0
    m.indexButtonBack = 1
    m.ADD_NEW_BUTTON_INDEX = 0

    m.loadingIndicator = CreateObject("roSGNode", "ProgressDialog")
    m.loadingIndicator.title = "Please wait..."

    m.lists = m.top.findNode("lists")
    m.header = m.top.findNode("header")
    m.header.observeField("focusIndex", "onHeaderFocusIndexChanged")
    m.layoutAnimation = m.top.findNode("layoutAnimation")
    m.portalLists = m.top.findNode("portalLists")
    m.portalactions = m.top.findNode("portalactions")

    m.actions = m.top.findNode("actions")

    m.portalLists.observeField("itemSelected", "OnListSelected")
    m.portalactions.observeField("itemSelected", "OnPortalActionSelected")
    m.lists.observeField("itemSelected", "OnListSelected")
    m.actions.observeField("itemSelected", "OnActionSelected")

    m.kbdialogLabel = getDialogLabel()
    m.kbdialogUrl = getDialogUrl()
    m.portalMode = false
    if (Instr(0, getSavedMode(),"list") = 0) m.header.focusIndex = 1
    m.top.observeField("focusedChild", "onFocusChange")
end function

function onHeaderFocusIndexChanged(event)
    m.portalMode = event.getData() <> 0
    moveLayout()
end function

function moveLayout()
    ' for i = 0 to m.layoutAnimation.getChildCount() - 1
    '     m.layoutAnimation.getChild(i).reverse = (index = 1)
    ' end for
    if (m.portalMode)
        m.layoutAnimation.getChild(0).keyValue = [[125, 120], [-1280, 120]]
        m.layoutAnimation.getChild(1).keyValue = [1, 0]
        m.layoutAnimation.getChild(2).keyValue = [[1280, 120], [125, 120]]
        m.layoutAnimation.getChild(3).keyValue = [0, 1]
    else
        m.layoutAnimation.getChild(0).keyValue = [[-1280, 120], [125, 120]]
        m.layoutAnimation.getChild(1).keyValue = [0, 1]
        m.layoutAnimation.getChild(2).keyValue = [[125, 120], [1280, 120]]
        m.layoutAnimation.getChild(3).keyValue = [1, 0]
    end if
    m.layoutAnimation.control = "start"
end function

function getDialogLabel() as Object
    kbdialogLabel = CreateObject("roSGNode", "BackKeyboardDialog")
    kbdialogLabel.id = "label"
    kbdialogLabel.title = "Please enter the playlist label"
    kbdialogLabel.text = ""
    kbdialogLabel.buttons = ["Continue", "Back"]
    kbdialogLabel.ObserveField("buttonSelected", "onDialogButtonSelected")
    kbdialogLabel.observeField("wasClosed", "oDialogWasClosed")

    return kbdialogLabel
end function

function getDialogUrl() as Object
    kbdialogUrl = CreateObject("roSGNode", "BackKeyboardDialog")
    kbdialogUrl.id = "url"
    kbdialogUrl.title = "Please enter the playlist url"
    kbdialogUrl.text = ""
    kbdialogUrl.buttons = ["Submit", "Back"]
    kbdialogUrl.ObserveField("buttonSelected", "onDialogButtonSelected")
    kbdialogUrl.observeField("wasClosed", "oDialogWasClosed")

    return kbdialogUrl
end function

function loadLists(params = invalid as Object) as Boolean
    m.lists.content = invalid
    showLoading()
    doRequest("getManageLists", sub(response)
        hideLoading()
        m.lists.content = response.content
        arr = getSavedMode().split("-")
        if (arr.count() = 2) 
            if (arr[0] = "list") markListAsDefault(arr[1])
        end if
    end sub , { baseURL: m.global.BASE_URL, email: getEmail() })

    return true
end function

function loadPortalLists(params = invalid as Object) as Boolean
    m.portalLists.content = invalid
    showLoading()
    doRequest("getPortalLists", sub(response)
        hideLoading()
        m.portalLists.content = response.content
        arr = getSavedMode().split("-")
        if (arr.count() = 2) 
            if (arr[0] = "portal") markSelectedPortalAsDefault(arr[1])
        end if
    end sub , { baseURL: m.global.BASE_URL, email: getEmail() })

    return true
end function

sub submitNewList(label as String, url as String)
    showLoading()
    doRequest("addNewList", sub(response)
        hideLoading()
        loadLists()
        resetDialogsText()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        label: label.Escape()
        encodedURL: url.Escape()
    })
end sub

function initContent()
    loadLists()
    loadPortalLists()
end function

sub submitNewPortal(content as object)
    showLoading()
    doRequest("addNewPortal", sub(response)
        hideLoading()
        loadPortalLists()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        name: content.name.Escape()
        host: content.host.Escape()
        port: content.port.Escape()
        username: content.username.Escape()
        password: content.password.Escape()
    })
end sub

sub updateList(id as String, label as String, url as String)
    showLoading()
    doRequest("updateList", sub(response)
        hideLoading()
        loadLists()
        resetDialogsText()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        id: id
        label: label.Escape()
        encodedURL: url.Escape()
    })
end sub

sub updatePortal(content)
    showLoading()
    doRequest("updatePortal", sub(response)
        hideLoading()
        loadPortalLists()
    end sub , {
        baseURL: m.global.BASE_URL,
        id: getCurrentSelectedPortalContent().id
        email: getEmail()
        name: content.name.Escape()
        host: content.host.Escape()
        port: content.port.Escape()
        username: content.username.Escape()
        password: content.password.Escape()
    })
end sub

sub refreshSelectedList(id as String)
    showLoading()
    doRequest("refreshList", sub(response)
        hideLoading()
        loadLists()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        id: id
    })
end sub

sub markListAsDefault(contentId = "") as void
    if (contentId <> "") markSelectedPortalAsDefault()
    listsContent = m.lists.content
    if (listsContent = invalid) return
    itemCount = listsContent.getChildCount()
    for i = 0 to itemCount - 1
        item = listsContent.getChild(i)
        if (item.id = contentId)
            item.title = item.title + " (default)"
            saveMode("list-" + contentId)
        else
            item.title = item.title.replace(" (default)", "")
        end if
    end for
end sub

sub deleteSelectedList(id as String)
    showLoading()
    doRequest("deleteList", sub(response)
        hideLoading()
        loadLists()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        id: id
    })
end sub

sub markSelectedPortalAsDefault(contentId = "") as void
    if (contentId <> "") markListAsDefault()
    portalListsContent = m.portalLists.content
    if (portalListsContent = invalid) return
    itemCount = portalListsContent.getChildCount()
    for i = 0 to itemCount - 1
        item = portalListsContent.getChild(i)
        if (item.id = contentId)
            item.title = item.title + " (default)"
            saveMode("portal-" + contentId)
        else   
            item.title = item.title.replace(" (default)", "")
        end if
    end for
end sub

sub deleteSelectedPortal(id as String)
    showLoading()
    doRequest("deletePortal", sub(response)
        hideLoading()
        loadPortalLists()
    end sub , {
        baseURL: m.global.BASE_URL,
        email: getEmail()
        id: id
    })
end sub

sub showLoading()
    if (m.top.visible) m.top.GetParent().dialog = m.loadingIndicator
end sub

sub hideLoading()
    if m.top.getScene().dialog <> invalid
        m.top.getScene().dialog.close = true
    end if
end sub

sub resetDialogsText()
    m.kbdialogLabel = getDialogLabel()
    m.kbdialogUrl = getDialogUrl()
end sub

sub OnListSelected()
    if (m.portalMode)
        m.portalActions.setFocus(true)
    else
        m.actions.setFocus(true)
    end if
end sub

sub onDialogButtonSelected(event as Object)
    node = event.getRoSGNode()
    buttonIndex = event.getData()

    if node.id = "label"
        if (buttonIndex = m.indexButtonGo)
            m.parentScene.dialog = m.kbdialogUrl
        else
            m.parentScene.dialog.close = true
            resetDialogsText()
        end if
    else if node.id = "url"
        m.parentScene.dialog.close = true
        if (buttonIndex = m.indexButtonGo)
            if (m.actions.itemFocused = m.ADD_NEW_BUTTON_INDEX)
                submitNewList(m.kbdialogLabel.text, m.kbdialogUrl.text)
            else
                content = getCurrentSelectedListContent()
                if (content <> invalid)
                    updateList(content.id, m.kbdialogLabel.text, m.kbdialogUrl.text)
                end if
            end if
        else
            resetDialogsText()
        end if
    end if
end sub

sub OnActionSelected(event as Object)
    if GetParentScene() = invalid then
        return
    end if

    if not m.global.isSubscribe
        showNonSubscribeNotification()
        return
    end if

    action = event.getData()

    if action = 0
        m.parentScene.dialog = m.kbdialogLabel
    else if action = 1
        content = getCurrentSelectedListContent()
        if (content <> invalid)
            m.kbdialogLabel.text = content.label
            m.kbdialogUrl.text = content.url
            m.parentScene.dialog = m.kbdialogLabel
        end if
    else if action = 2
        content = getCurrentSelectedListContent()
        if (content <> invalid)
           refreshSelectedList(content.id)
        end if
    else if action = 4
        content = getCurrentSelectedListContent()

        if (content <> invalid)
            deleteSelectedList(content.id)
        end if
    else if action = 3
        content = getCurrentSelectedListContent()

        if (content <> invalid)
            markListAsDefault(content.id)
        end if
    end if
end sub

function onAddPortalDialogClosed(event=invalid) as void
    m.mainScene.removeChild(m.addPortalDialog)
    m.portalactions.setFocus(true)
end function

function onAddPortalDialogSubmited(event) as void
    content = event.getData()
    submitNewPortal(content)
    onAddPortalDialogClosed()
end function

function onUpdatePortalDialogSubmited(event) as void
    content = event.getData()
    updatePortal(content)
    onAddPortalDialogClosed()
end function

sub OnPortalActionSelected(event as Object)
    if GetParentScene() = invalid then
        return
    end if

    if not m.global.isSubscribe
        showNonSubscribeNotification("portals")
        return
    end if

    action = event.getData()

    if action = 0
        m.mainScene = m.top.getScene()
        m.addPortalDialog = m.mainScene.createChild("AddPortalDialog")
        m.addPortalDialog.observeField("content", "onAddPortalDialogSubmited")
        m.addPortalDialog.observeField("close", "onAddPortalDialogClosed")
        m.addPortalDialog.setFocus(true)
    else if action = 1
        content = getCurrentSelectedPortalContent()
        if (content <> invalid)
            m.mainScene = m.top.getScene()
            m.addPortalDialog = m.mainScene.createChild("AddPortalDialog")
            m.addPortalDialog.content = {
                name: content.name
                host: content.host
                port: content.port
                username: content.username
                password: content.password
            }
            m.addPortalDialog.observeField("content", "onUpdatePortalDialogSubmited")
            m.addPortalDialog.observeField("close", "onAddPortalDialogClosed")
            m.addPortalDialog.setFocus(true)
        end if
    else if action = 2
        content = getCurrentSelectedPortalContent()
        if (content <> invalid)
            markSelectedPortalAsDefault(content.id)
        end if
    else if action = 3
        content = getCurrentSelectedPortalContent()

        if (content <> invalid)
            deleteSelectedPortal(content.id)
        end if
    end if
end sub

sub showNonSubscribeNotification(text="playlists")
    dialogBillingNA = CreateObject("roSGNode", "BackDialog")
    dialogBillingNA.title = "Activate Your Lifetime Premium Access!"
    dialogBillingNA.message = "To manage your "+text+", you have to Activate Your Lifetime Premium Access. Under info click <Activate> button and follow the steps on the screen if you have already signed up."
    dialogBillingNA.buttons = ["OK"]
    dialogBillingNA.ObserveField("buttonSelected", "OnNoSubscriptionAlertClick")

    m.parentScene.dialog = dialogBillingNA
end sub


sub showEmptyListsMessage()
    dialogBillingNA = CreateObject("roSGNode", "BackDialog")
    dialogBillingNA.title = "Empty Playlist"
    dialogBillingNA.message = "Please select a playlist!"
    dialogBillingNA.buttons = ["OK"]
    dialogBillingNA.ObserveField("buttonSelected", "OnNoSubscriptionAlertClick")

    m.parentScene.dialog = dialogBillingNA
end sub

sub showEmptyPortalListsMessage()
    dialogBillingNA = CreateObject("roSGNode", "BackDialog")
    dialogBillingNA.title = "Empty Portal Lists"
    dialogBillingNA.message = "Please select a Portal!"
    dialogBillingNA.buttons = ["OK"]
    dialogBillingNA.ObserveField("buttonSelected", "OnNoSubscriptionAlertClick")

    m.parentScene.dialog = dialogBillingNA
end sub

sub OnNoSubscriptionAlertClick()
    m.parentScene.dialog.close = true
end sub

function getCurrentSelectedListContent() as Object
    selectedIndex = m.lists.itemFocused

    if (m.lists.content = invalid)
        showEmptyListsMessage()
        return invalid
    end if
    return m.lists.content.getChild(selectedIndex)
end function

function getCurrentSelectedPortalContent() as Object
    selectedIndex = m.portalLists.itemFocused

    if (m.portalLists.content = invalid)
        showEmptyPortalListsMessage()
        return invalid
    end if
    return m.portalLists.content.getChild(selectedIndex)
end function

sub onFocusChange()
    if (m.top.hasFocus()) m.header.setFocus(true)
end sub

function onKeyEvent(key as String, press as Boolean) as Boolean
    if press
        lists = m.lists
        actions = m.actions
        if (m.portalMode)
            lists = m.portalLists
            actions = m.portalactions
        end if
        if key = "left" and actions.hasFocus()
            lists.setFocus(true)
            return true
        else if key = "right" and lists.hasFocus()
            actions.setFocus(true)
            return true
        else if key = "down" and lists.content = invalid
            actions.setFocus(true)
            return true
        else if key = "down" and m.header.isInFocusChain()
            lists.setFocus(true)
            return true
        else if key = "up" and (not m.header.isInFocusChain())
            m.header.setFocus(true)
            return true
        end if
    end if
    return false
end function
