sub init()
    m.name = m.top.findNode("name")
    m.host = m.top.findNode("host")
    m.port = m.top.findNode("port")
    m.userName = m.top.findNode("userName")
    m.password = m.top.findNode("password")
    m.createButton = m.top.findNode("createButton")
    m.backButton = m.top.findNode("backButton")
    m.errorLabel = m.top.findNode("errorLabel")

    m.name.observeField("selected", "showKeyBoard")
    m.host.observeField("selected", "showKeyBoard")
    m.port.observeField("selected", "showKeyBoard")
    m.userName.observeField("selected", "showKeyBoard")
    m.password.observeField("selected", "showKeyBoard")
    m.createButton.observeField("selected", "onCreateButtonSelected")
    m.backButton.observeField("selected", "onBackButtonSelected")

    m.lastContent = {}
    m.nameText = ""
    m.hostText = ""
    m.portText = ""
    m.userNameText = ""
    m.passwordText = ""
    m.focusArr = [m.name, m.host, m.port, m.userName, m.password, m.createButton]
    m.focusIndex = 0

    m.top.observeField("focusedChild", "onFocusedChildChanged")
    m.top.findNode("background").color = m.global.appColor
end sub

sub setupContent() as void
    content = m.top.content
    if (content = invalid) return
    m.nameText = content.name
    m.hostText = content.host
    m.portText = content.port
    m.userNameText = content.username
    m.passwordText = content.password
    m.createButton.text = "Update"
    updateUIText()
end sub

sub onCreateButtonSelected()
    text = "Please input "
    portText = m.portText
    if (m.portText = "") portText = "80"

    if m.nameText = ""
        m.errorLabel.text = text + "Portal Name"
    else if m.hostText = ""
        m.errorLabel.text = text + "Host"
    else if m.userNameText = ""
        m.errorLabel.text = text + "Username"
    else if m.passwordText = ""
        m.errorLabel.text = text + "Password"
    else 
        m.top.content = {
            name: m.nameText
            host: m.hostText
            port: portText
            username: m.userNameText
            password: m.passwordText
        }
    end if
end sub

sub onBackButtonSelected()
    m.top.close = true
end sub

sub onFocusedChildChanged()
    if m.top.hasFocus()
        updateUIText()
    end if
end sub

sub showKeyBoard(event)
    node = event.getRoSGNode()
    nodeId = node.id
    hintText = ""
    currentText = m.nameText
    if (nodeId = "name") hintText = "Portal Name"
    if (nodeId = "host") 
        hintText = "Host"
        currentText = m.hostText
    end if
    if (nodeId = "port") 
        hintText = "Port"
        currentText = m.portText
    end if
    if (nodeId = "username") 
        hintText = "Username"
        currentText = m.usernameText
    end if
    if (nodeId = "password") 
        hintText = "Password"
        currentText = m.passwordText
    end if 

    m.keyboard = m.top.createChild("KeyboardControl")
    m.keyboard.hintText = hintText
    m.keyboard.text = currentText
    m.keyboard.unObserveField("close")
    m.keyboard.observeField("close", "onKeyBoardClosed")
    m.keyboard.setFocus(true)
end sub

sub onKeyBoardClosed() as void
    if (m.keyboard = invalid) return
    
    nodeId = m.focusArr[m.focusIndex].id
    if (m.keyboard.close = false)
        m.keyboard.text = ""
    else
        m.focusIndex += 1
    end if

    if (m.keyboard.text <> "")
        if (nodeId = "name") m.nameText = m.keyboard.text
        if (nodeId = "host") m.hostText = m.keyboard.text
        if (nodeId = "port") m.portText = m.keyboard.text
        if (nodeId = "username") m.usernameText = m.keyboard.text
        if (nodeId = "password") m.passwordText = m.keyboard.text
    end if

    m.top.removeChild(m.keyboard)
    m.keyboard = invalid
    updateUIText()
end sub

function updateUIText()
    if (m.nameText = "")
        m.name.text = "Portal Name" 
        m.name.textColor = "#ffffff88"
    else
        m.name.text = m.nameText
        m.name.textColor = "#ffffff"
    end if

    if (m.hostText = "")
        m.host.text = "Host" 
        m.host.textColor = "#ffffff88"
    else
        m.host.text = m.hostText
        m.host.textColor = "#ffffff"
    end if

    if (m.portText = "")
        m.port.text = "Port - leave it empty if not provided" 
        m.port.textColor = "#ffffff88"
    else
        m.port.text = m.portText
        m.port.textColor = "#ffffff"
    end if

    if (m.usernameText = "")
        m.username.text = "Username" 
        m.username.textColor = "#ffffff88"
    else
        m.username.text = m.usernameText
        m.username.textColor = "#ffffff"
    end if

    if (m.passwordText = "")
        m.password.text = "Password" 
        m.password.textColor = "#ffffff88"
    else
        passwordTextSecure = ""
        for i=0 to len(m.passwordText) - 1
            passwordTextSecure += "*"
        end for

        m.password.text = passwordTextSecure
        m.password.textColor = "#ffffff"
    end if

    m.focusArr[m.focusIndex].setFocus(true)
end function

function onKeyEvent(key as string, isPressed as boolean) as boolean
    handled = false

    if isPressed
        if key = "back"
            m.top.close = true
        else if key = "up"
            m.focusIndex--
            
            if (m.focusIndex < 0) m.focusIndex = 0
            m.focusArr[m.focusIndex].setFocus(true)
            handled = true
        else if key = "down"
            m.focusIndex++
            if (m.focusIndex > m.focusArr.count() - 1) m.focusIndex = m.focusArr.count() - 1
            m.focusArr[m.focusIndex].setFocus(true)
            handled = true
        else if key = "left"
            if (m.createButton.isInFocusChain()) m.backButton.setFocus(true)
            handled = true
        else if key = "right"
            if (m.backButton.isInFocusChain()) m.createButton.setFocus(true)
            handled = true
        end if
    end if

    return handled
end function