sub init()
    m.videoLabelList = m.top.findNode("videoLabelList")
    m.videoLabelList.observeField("itemSelected", "onVideoLabelListSelected")
    m.videoListGroup = m.top.findNode("videoListGroup")
    m.video = m.top.findNode("selectedVideo")
    m.video.observeField("state", "controlvideoplay")
    m.videoLoadingIndicator = m.top.findNode("videoLoadingIndicator")
    m.loadingIndicator = m.top.findNode("loadingIndicator")
    m.actionButtonGroup = m.top.findNode("actionButtonGroup")
    m.categoryList = m.top.findNode("categoryList")
    m.categoryList.observeField("itemSelected", "onCategoryListItemSelected")
    m.top.observeField("focusedChild", "onFocusedChildChanged")
    m.msgDialog = m.top.findNode("backDialog")
    m.actionButtonGroup.observeField("buttonSelected", "onActionButtonSelected")
    m.searchButton = m.top.findNode("searchButton")
    m.searchKeyboard = m.top.findNode("searchKeyboard")
    m.searchButton.observeField("buttonSelected", "showSearchKeyboard")
    m.searchTextBox = m.searchKeyboard.textEditBox
    m.searchTextBox.observeField("text", "onSearchInputChange")
    m.labelListLoadingIndicator = m.top.findNode("labelListLoadingIndicator")
    ToggleScreen("small")
    m.videoGroup = m.top.findNode("videoGroup")
    m.index = 0
    m.initContent = false
end sub

sub onFocusedChildChanged()
    if (m.top.hasFocus())
        if (m.initContent = false)
            initContent()
        else
            m.categoryList.setFocus(true)
        end if
    end if
end sub

function onVideoLabelListSelected()
    playvideo()
end function

function initContent()
    m.initContent = true
    m.loadingIndicator.visible = true
    doRequest("getHomeContent", sub(response)
        m.categoryList.content = response.content
        m.loadingIndicator.visible = false
        if (m.top.hasFocus()) m.categoryList.setFocus(true)
    end sub, {baseURL: m.global.BASE_URL, email: getEmail()})
    m.videoListGroup.visible = false
end function

function showSearchKeyboard()
    m.searchKeyboard.setFocus(true)
    m.searchKeyboard.visible= true
    m.videoGroup.visible = false
end function

function onSearchInputChange(event)
    searchTerm = event.getData()

    if(Len(searchTerm) >= 0)
        m.videoGroup.visible = false
        searchQuery(searchTerm)
        m.searchButton.text = searchTerm
    end if
end function

function onActionButtonSelected(event)
    selectedIndex = event.getData()
    actionType = m.actionButtonGroup.getChild(selectedIndex).text
    if (actionType = "Play")
        ToggleScreen("full")
    else if (actionType = "Add to Favourites")
        m.loadingIndicator.visible = true
        favoriteAction("addFavourite")
    else if (actionType = "Remove from Favourites")
        m.loadingIndicator.visible = true
        favoriteAction("removeFromFavorites")
    end if
end function

function favoriteAction(action as string)
    doRequest(action, sub(response)
        if (response.status = "success")
            if (m.focusVideocontent.isFavourite = true)
                m.actionButtonGroup.buttons = ["Add to Favourites"]
                m.focusVideocontent.isFavourite = false
            else
                m.actionButtonGroup.buttons = ["Remove from Favourites"]
                m.focusVideocontent.isFavourite = true
            end if

            m.loadingIndicator.visible = false
            m.actionButtonGroup.setFocus(true)
            m.video.control = "play"
        else
            showFavoriteErrorDialog(response.message)
        end if
    end sub, {  
        baseURL: m.global.BASE_URL,
        email: getEmail(),
        channelID: m.focusVideocontent.title
    })
end function

sub showFavoriteErrorDialog(message as String)
    m.lastFocusNode = m.actionButtonGroup
    m.msgDialog.buttons = ["OK"]
    m.msgDialog.buttonGroup.focusBitmapUri = "pkg:/images/button_on.9.png"
    m.msgDialog.ObserveField("buttonSelected", "onErrorDialogDismiss")
    m.msgDialog.ObserveField("isBackPressed", "onErrorDialogDismiss")
    m.msgDialog.title = "Error"
    m.msgDialog.message = message
    m.msgDialog.visible = true
    m.msgDialog.setFocus(true)
end sub

sub controlvideoplay(event as Object)
    state = event.getData()
    if state = "buffering"
        m.video.enableUI = false
        m.videoLoadingIndicator.visible = true
    else if state = "playing"
        if (m.mode = "full")
            m.video.enableUI = true
        end if    
        m.videoLoadingIndicator.visible = false
    else if (state = "finished" AND m.mode = "full")
        m.videoLabelList.setFocus(true)
        ToggleScreen("small")
    else if state = "error"
    end if
end sub

sub playvideo(playingIndex = -1 as integer) as void
    if (NOT m.global.isSubscribe) 
        showNonSubscribeNotification()
        return
    end if
    focusNode = true
    if playingIndex = -1
        m.index = m.videoLabelList.itemFocused
    else
        m.index = playingIndex
        focusNode = false
    end if
    content = m.videoLabelList.content
    if (m.index = - 1 or content = invalid ) return
    m.focusVideocontent = m.videoLabelList.content.getChild(m.index)
    
    if (m.global.selectedVideoId <> m.focusVideocontent.id)
        ?"playcontent: ", m.focusVideocontent.url
        m.focusVideocontent.streamFormat = "auto"
        m.video.control = "stop"
        m.video.content = m.focusVideocontent
        m.video.globalCaptionMode = "On"
        m.video.control = "play"
        m.video.visible = true
        if (m.focusVideocontent.isFavourite = true)
            m.actionButtonGroup.buttons = ["Remove from Favourites"]
        else
            m.actionButtonGroup.buttons = ["Add to Favourites"]
        end if
        ToggleScreen("small")
    else
        ToggleScreen("full")
    end if
    m.global.selectedVideoId = m.focusVideocontent.id
end sub

function ToggleScreen(mode)
    m.mode = mode
    if m.mode = "small" then
        m.video.scale = [0.36, 0.36]
        m.video.translation = [460, 106]
        m.videoLoadingIndicator.width = 547
        m.videoLoadingIndicator.height = 309
        m.videoLoadingIndicator.translation= [417, 90]
        m.video.inheritParentTransform = true
        m.videoLoadingIndicator.inheritParentTransform = true
        m.actionButtonGroup.visible = true 
        m.searchButton.visible = true
    else
        m.videoLoadingIndicator.width = 1280
        m.videoLoadingIndicator.height = 720
        m.videoLoadingIndicator.translation= [0,0]
        m.video.scale = [1, 1]
        m.video.translation = [0, 0]
        m.video.setFocus(true)
        m.video.enableUI = true
        m.video.inheritParentTransform = false
        m.videoLoadingIndicator.inheritParentTransform = false
        m.actionButtonGroup.visible = false
        m.searchButton.visible = false
    end if
end function

sub showNonSubscribeNotification()
    m.top.dialog = dialogBillingNA
    m.lastFocusNode = m.videoLabelList
    m.msgDialog.title = "Activate Your Lifetime Premium Access!"
    m.msgDialog.message = "To watch the video, Please Activate Your Lifetime Premium Access. Under info click <Activate Your Lifetime Premium Access> button and follow the steps on the screen if you have already signed up."
    m.msgDialog.buttons = ["OK"]
    m.msgDialog.ObserveField("buttonSelected", "onDismiss")
    m.msgDialog.ObserveField("isBackPressed", "onDismiss")
    m.msgDialog.visible = true
    m.msgDialog.setFocus(true)
end sub

function onCategoryListItemSelected(event)
    getVideoList(m.categoryList.content.getChild(event.getData()).id)
end function

function getVideoList(cId as String)
    m.loadingIndicator.visible = true
    doRequest("getVideoContent", sub(response)
        m.allVideoList = response.videoItems
        showvideolist(response.content)
    end sub, {
        baseURL: m.global.BASE_URL
        email: getEmail()
        videoId: cId
    })
end function

sub showvideolist(content as Object)
    m.loadingIndicator.visible = false
    if (content.getChildCount() = 0)
        showNoVideoDialog()
        return
    end if
    m.lastVideoLabelListContent = content
    m.videoLabelList.content = content
    m.videoLabelList.setFocus(true)
    m.videoListGroup.visible = true
    ' play the first video in the list
    if (m.global.isSubscribe)
        m.global.selectedVideoId = ""
        playvideo(0)
    end if    
end sub

sub showNoVideoDialog()
    m.lastFocusNode = m.categoryList
    m.msgDialog.buttons = ["OK"]
    m.msgDialog.buttonGroup.focusBitmapUri = "pkg:/images/button_on.9.png"
    m.msgDialog.ObserveField("buttonSelected", "onDismiss")
    m.msgDialog.ObserveField("isBackPressed", "onDismiss")
    m.msgDialog.title = "No video in this category"
    m.msgDialog.visible = true
    m.msgDialog.setFocus(true)
end sub

sub onDismiss()
    m.msgDialog.visible = false
    m.lastFocusNode.setFocus(true)
end sub

function searchQuery(searchString)
    m.searchContentExist = false
    m.labelListLoadingIndicator.visible = true
    m.videoListGroup.visible = false
    if (m.searchTask <> invalid)
        m.searchTask.control = "STOP"
    end if
    
    if searchString = ""
        if (m.lastVideoLabelListContent <> invalid) m.videoLabelList.content = m.lastVideoLabelListContent
        m.searchButton.text = "Search"
        if searchString = "" then m.searchContentExist = true
        m.videoListGroup.visible = true
        m.labelListLoadingIndicator.visible = false
    else
        m.searchTask = doRequest("search", sub(response)
            m.searchContentExist = response.content <> invalid AND response.content.getChildCount() > 0
            m.videoLabelList.content = response.content
            m.videoListGroup.visible = true
            m.labelListLoadingIndicator.visible = false
        end sub, {
            baseURL: m.global.BASE_URL,
            email: getEmail(),
            keyword: searchString
        })
    end if
end function

function onKeyEvent(key as string, isPressed as boolean) as boolean
    if isPressed
        if key = "back"
            if (m.categoryList.isInFocusChain() = true)
                m.video.control = "stop"
                return false
            else if (m.mode <> "small" AND m.actionButtonGroup.isInFocusChain() = false)
                ToggleScreen("small")
                m.videoLabelList.setFocus(true)
            else if (m.actionButtonGroup.isInFocusChain() = true)
                m.actionButtonGroup.focusButton = 0
                m.videoLabelList.setFocus(true)
            else if (m.videoLabelList.isInFocusChain() = true)
                m.categoryList.setFocus(true)
            else
                m.video.control = "stop"
                return false
            end if
        else if (key = "left")
            if (m.searchKeyboard.isInFocusChain())
                m.searchKeyboard.visible = false
                m.searchButton.setFocus(true)
                m.videoGroup.visible = true
                ' play the first video in the list
                if (m.global.isSubscribe)
                    m.global.selectedVideoId = ""
                    playvideo(0)
                end if   
            else if (m.actionButtonGroup.isInFocusChain())
                m.actionButtonGroup.focusButton = 0
                m.videoLabelList.setFocus(true)
            else if m.videoLabelList.isInFocusChain() OR m.searchButton.isInFocusChain()
                m.categoryList.setFocus(true)
            end if
        else if (key = "right")
            if (m.categoryList.isInFocusChain()) 
                if (m.videoListGroup.visible) 
                    m.videoLabelList.setFocus(true)
                else
                    m.searchButton.setFocus(true)
                end if
            else if m.videoLabelList.isInFocusChain()
                m.videoLabelList.jumpToItem = m.index
                m.actionButtonGroup.setFocus(true)
            end if
        else if (key = "up")
            if (m.videoLabelList.isInFocusChain()) 
                m.searchButton.setFocus(true)
            else if (m.categoryList.isInFocusChain())
                m.video.control = "stop"
                return false
            end if
        else if (key = "down")
            if (m.searchButton.isInFocusChain()) 
                m.videoLabelList.setFocus(true)
            end if
        end if
    end if

    return true
end function