sub init()
    m.videoLabelList = m.top.findNode("videoLabelList")
    m.videoLabelList.observeField("itemSelected", "onVideoLabelListSelected")
    m.video = m.top.findNode("selectedVideo")
    m.video.observeField("state", "controlvideoplay")
    m.videoLoadingIndicator = m.top.findNode("videoLoadingIndicator")
    m.loadingIndicator = m.top.findNode("loadingIndicator")
    m.actionButtonGroup = m.top.findNode("actionButtonGroup")

    m.top.observeField("focusedChild", "onFocusedChildChanged")

    m.actionButtonGroup.observeField("buttonSelected", "onActionButtonSelected")
    m.portalTypeConstants = {
        "live": "live",
        "movie": "vods",
        "series": "series",
        "vods": "vods",
    }
    m.videoGroup = m.top.findNode("videoGroup")
    m.rowlist = m.top.findNode("rowlist")
    m.rowlist.observeField("itemSelected", "onRowListItemSelected")
    m.seriesControl = m.top.findNode("seriesControl")
    m.allEPSItem = {}
end sub

function onVideoLabelListSelected()
    playvideo()
end function

function ResetControl()
    ToggleScreen("small")
    m.videoGroup.visible = false
    m.rowlist.visible = false
    m.videoLabelList.itemSize = [320, 50]
    m.index = 0
    m.video.control = "stop"
end function

function onRowListItemSelected()
    selectedIndex = m.rowlist.rowItemSelected
    itemSelected = m.rowlist.content.getChild(selectedIndex[0]).getChild(selectedIndex[1])
    seasonNumber = itemSelected.season_number.toStr()
    seriesContent = m.allEPSItem[m.series_id][seasonNumber]
    
    if (seriesContent <> invalid )
        m.seriesControl.visible = true
        m.seriesControl.content = seriesContent
        title = m.currentStream.title
        titleArr = m.currentStream.title.split(" - ")
        if (titleArr.count() > 1) title = titleArr[1]
        m.seriesControl.seasonInfo = {
            seriesId: m.series_id,
            seasonNumber: seasonNumber
            cover: itemSelected.cover
            title: m.top.categoryname + " > " + title + " - " + itemSelected.name
        }
        m.seriesControl.setFocus(true)
    else
        showNoVideoDialog()
    end if

end function

sub showNoVideoDialog(lastFocusNode = m.rowlist)
    m.lastFocusNode = lastFocusNode
    m.msgDialog = CreateObject("roSGNode", "BackDialog")
    m.msgDialog.buttons = ["OK"]
    m.msgDialog.buttonGroup.focusBitmapUri = "pkg:/images/button_on.9.png"
    m.msgDialog.ObserveField("buttonSelected", "onDismiss")
    m.msgDialog.ObserveField("isBackPressed", "onDismiss")
    m.msgDialog.title = "No video in this category"
    m.top.appendChild(m.msgDialog)
    m.msgDialog.setFocus(true)
end sub

sub onDismiss()
    if (m.msgDialog <> invalid)
        m.top.removeChild(m.msgDialog)
        m.msgDialog = invalid
    end if
    m.lastFocusNode.setFocus(true)
end sub

function onActionButtonSelected(event)
    selectedIndex = event.getData()
    actionType = m.actionButtonGroup.getChild(selectedIndex).text
    if (actionType = "Play")
        ToggleScreen("full")
    else if (actionType = "Add to Favourites")
        m.loadingIndicator.visible = true
        favoriteAction("add")
    else if (actionType = "Remove from Favourites")
        m.loadingIndicator.visible = true
        favoriteAction("remove")
    end if
end function

function favoriteAction(action as string)
    if (m.currentStream = invalid) m.currentStream = m.videoLabelList.content.getChild(0)
    doRequest("portalFavourites", sub(response)
        if (response.status = "success")
            portalType = m.portalTypeConstants[m.currentStream["stream_type"]]
            portalFavourites = m.global.portalFavourites
            if (m.actionButtonGroup.getChild(0).text = "Add to Favourites")
               portalFavourites[portalType][m.currentStream.id] = response.status
                m.actionButtonGroup.buttons = ["Remove from Favourites"]
            else if (m.actionButtonGroup.getChild(0).text = "Remove from Favourites")
                m.actionButtonGroup.buttons = ["Add to Favourites"]
                portalFavourites[portalType][m.currentStream.id] = invalid
            end if

            m.global.portalFavourites = portalFavourites
            m.loadingIndicator.visible = false
            m.actionButtonGroup.setFocus(true)
        end if
    end sub, {
        baseURL: m.global.BASE_URL,
        token: m.global.token,
        portalID: m.global.defaultPortal.id,
        portalType: m.portalTypeConstants[m.currentStream["stream_type"]],
        streamID: m.currentStream.id,
        action: action
        streamName: m.currentStream.title.Escape()
        steamLogo: m.currentStream.logo.Escape()
    })
end function

sub controlvideoplay(event as object)
    state = event.getData()
    if state = "buffering"
        m.video.enableUI = false
        m.videoLoadingIndicator.visible = true
    else if state = "playing"
        if (m.mode = "full")
            m.video.enableUI = true
        end if
        m.videoLoadingIndicator.visible = false
    else if (state = "finished")
        m.videoLabelList.setFocus(true)
        ToggleScreen("small")
    else if state = "error"
    end if
end sub

sub setupContent() as void
    content = m.top.content
    if (content = invalid) return
    m.videoLabelList.content = content
    m.defaultPortal = m.global.defaultPortal
    m.global.selectedVideoId = ""
    if (NOT m.top.seriesFavoriteMode) 
        if (m.top.autoPlay) 
            playvideo(0)
        else
            ToggleScreen("small")
            m.videoGroup.visible = false
            m.rowlist.visible = false
        end if
        m.videoLabelList.itemSize = [320, 50]
    else
        ToggleScreen("small")
        m.videoGroup.visible = false
        m.rowlist.visible = false
        m.videoLabelList.itemSize = [500, 50]
    end if
end sub

sub playvideo(playingIndex = -1 as integer) as void
    m.rowlist.content = invalid
    if playingIndex = -1
        m.index = m.videoLabelList.itemFocused
    else
        m.index = playingIndex
    end if
    if (m.index = -1) return
    m.currentStream = m.videoLabelList.content.getChild(m.index)
    if (m.currentStream = invalid) return

    if (m.currentStream.id <> "" AND m.currentStream.series_id = invalid)
        m.videoGroup.visible = true
        m.rowlist.visible = false
        if (m.global.selectedVideoId <> m.currentStream.id)
            m.videocontent = CreateObject("roSGNode", "ContentNode")
            if (m.defaultPortal.port = "") m.defaultPortal.port = "80"
            sufix = ""

            portalType = m.portalTypeConstants[m.currentStream["stream_type"]]
            baseVideoUrl = Substitute("{0}:{1}/{2}/{3}", m.defaultPortal.host, m.defaultPortal.port, m.defaultPortal.username, m.defaultPortal.password)
            if (portalType = "vods" or portalType = "series")
                baseVideoUrl = Substitute("{0}/{1}/{2}/{3}", m.defaultPortal.host + ":" + m.defaultPortal.port, m.currentStream.stream_type, m.defaultPortal.username, m.defaultPortal.password)
                if m.currentStream.container_extension <> invalid
                    sufix = "." + m.currentStream.container_extension
                else
                    sufix = ".mkv"
                end if
            end if
            m.videocontent.update({
                id: m.currentStream.id
                title: m.currentStream.TITLE
                url: Substitute("{0}/{1}{2}", baseVideoUrl, m.currentStream.id, sufix)
            }, true)

            ?"playcontent: ", m.currentStream.stream_type, m.videocontent.url

            m.videocontent.streamFormat = "auto"
            m.video.control = "stop"
            m.video.content = m.videocontent
            m.video.globalCaptionMode = "On"
            m.video.control = "play"
            m.video.visible = true
            if (m.global.portalFavourites[portalType][m.currentStream.id] <> invalid)
                m.actionButtonGroup.buttons = ["Remove from Favourites"]
            else
                m.actionButtonGroup.buttons = ["Add to Favourites"]
            end if
            ToggleScreen("small")
        else
            if (m.global.selectedVideoId <> invalid)
                ToggleScreen("full")
            end if
        end if
        m.global.selectedVideoId = m.currentStream.id
    end if

    if (m.currentStream.series_id <> invalid)
        if (m.video.control <> "stop") m.video.control = "stop"
        ToggleScreen("small")
        m.videoGroup.visible = false
        m.series_id = m.currentStream.series_id.toStr()
        if (m.top.seriesFavoriteMode)
            m.rowlist.visible = false
        else
            m.rowlist.visible = true
            m.global.selectedVideoId = m.series_id
        end if

        if (m.allEPSItem[m.series_id] = invalid)
            m.allEPSItem[m.series_id] = { seasons: m.currentStream.seasons }
            m.loadingIndicator.visible = true
            defaultPortal = m.global.defaultPortal
            streamParams = {
                host: defaultPortal.host
                port: defaultPortal.port
                username: defaultPortal.username
                password: defaultPortal.password
            }
            streamParams.action = "get_series_info&series_id=" + m.series_id
            doRequest("getSeriesInfo", sub(response)
                m.loadingIndicator.visible = false
                if (response.content <> invalid)
                    seasons = m.allEPSItem[m.series_id].seasons
                    if (seasons = invalid) seasons = response.content.seasons
                    if (seasons = invalid) 
                        if (m.top.seriesFavoriteMode) showFavoritesSeriesContent()
                        return
                    end if
                        
                    if ((type(seasons) = "roArray"))
                        seasonsTemp = seasons
                        seasons = {}
                        for each season in seasonsTemp
                            if ((type(season) = "roAssociativeArray") AND season.season_number <> invalid)
                                seasons[season.season_number.toStr()] = season
                            end if
                        end for
                    end if

                    m.allEPSItem[m.series_id] = response.content
                    for each seasonNumber in response.content
                        if (seasonNumber <> "default" AND seasonNumber <> "seasons")
                            if (NOT seasons.doesExist(seasonNumber))
                                seasonData = {}             
                                seasonData.name = "Season " + seasonNumber
                                seasonData.season_number = seasonNumber
                                if (response.content["default"] <> invalid) seasonData.cover = response.content["default"].cover
                                seasons[seasonNumber] = seasonData
                            end if
                        end if
                        
                    end for
                    m.allEPSItem[m.series_id].seasons = seasons

                    if (m.top.seriesFavoriteMode)
                        showFavoritesSeriesContent()
                    else
                        updateRowListContent(seasons)
                    end if
                end if
            end sub, streamParams)
        else
            if (NOT m.top.seriesFavoriteMode)
                updateRowListContent(m.allEPSItem[m.series_id].seasons)
            else
                m.rowlist.visible = false
                showFavoritesSeriesContent()
            end if
        end if
    end if
end sub

function showFavoritesSeriesContent() as void
    if (m.currentStream.stream_id <> invalid)
        streamIdArr = m.currentStream.stream_id.split("-")
        seasonNumber = ""
        if (streamIdArr.count() > 1)
            seasonNumber = streamIdArr[streamIdArr.count() - 1]
        end if
        if (seasonNumber <> "")
            seriesContent = m.allEPSItem[m.series_id][seasonNumber]
            if (seriesContent <> invalid)
                m.rowlist.visible = false
                m.seriesControl.visible = true
                m.seriesControl.content = seriesContent
                m.seriesControl.seasonInfo = {
                    seriesId: m.series_id,
                    seasonNumber: seasonNumber
                    cover: m.currentStream.logo
                    title: m.currentStream.title
                }
                m.seriesControl.setFocus(true)
                return
            else
                
            end if
        end if
        showNoVideoDialog(m.videoLabelList)
    end if
end function

function updateRowListContent(seasons) as void
    if (seasons = invalid OR type(seasons) <> "roAssociativeArray") return
    m.rowlist.visible = true
    k = 0
    rowListContent = CreateObject("roSGNode", "ContentNode")
    row = rowListContent.createChild("ContentNode")
    for each seasonNumber in seasons.Keys()
        if (k = 3)
            k = 0
            row = rowListContent.createChild("ContentNode")
        end if
        if (k < 3)
            item = row.createChild("ContentNode")
            item.update(seasons[seasonNumber], true)
        end if
        k += 1
    end for
    m.rowlist.content = rowListContent
    if (rowListContent.getChildCount() > 0) m.rowlist.jumpToItem = [0, 0]
end function

function ToggleScreen(mode)
    m.mode = mode
    if m.mode = "small" then
        m.video.scale = [0.36, 0.36]
        m.video.translation = [460, 106]
        m.video.enableUI = false
        m.videoLoadingIndicator.width = 547
        m.videoLoadingIndicator.height = 309
        m.videoLoadingIndicator.translation = [417, 90]
        m.video.inheritParentTransform = true
        m.videoLoadingIndicator.inheritParentTransform = true
        m.actionButtonGroup.visible = true
    else
        m.videoLoadingIndicator.width = 1280
        m.videoLoadingIndicator.height = 720
        m.videoLoadingIndicator.translation = [0, 0]
        m.video.scale = [1, 1]
        m.video.translation = [0, 0]
        m.video.setFocus(true)
        m.video.enableUI = true
        m.video.inheritParentTransform = false
        m.videoLoadingIndicator.inheritParentTransform = false
        m.actionButtonGroup.visible = false
    end if
end function

sub onFocusedChildChanged()
    if m.top.hasFocus() and m.actionButtonGroup.hasFocus() = false
        m.videoLabelList.setFocus(true)
    end if
end sub

function onKeyEvent(key as string, isPressed as boolean) as boolean
    if isPressed
        if key = "back"
            if (m.mode <> "small" and m.actionButtonGroup.isInFocusChain() = false)
                ToggleScreen("small")
                m.videoLabelList.setFocus(true)
            else if (m.actionButtonGroup.isInFocusChain() = true)
                m.actionButtonGroup.focusButton = 0
                m.videoLabelList.setFocus(true)
            else if (m.seriesControl.isInFocusChain())
                m.seriesControl.visible = false
                if (m.rowlist.visible)
                    m.rowlist.setFocus(true)
                else
                    m.videoLabelList.setFocus(true)
                end if
            else
                m.video.control = "stop"
                return false
            end if
        else if (key = "left")
            if m.actionButtonGroup.isInFocusChain()
                m.videoLabelList.setFocus(true)
            else if m.rowlist.isInFocusChain()
                m.videoLabelList.setFocus(true)
            else
                if (NOT m.top.seriesFavoriteMode) m.videoLabelList.jumpToItem = m.index
                return false
            end if
        else if (key = "right")
            if (m.videoLabelList.isInFocusChain())
                if (NOT m.top.seriesFavoriteMode) m.videoLabelList.jumpToItem = m.index
                if (m.rowlist.visible)
                    m.rowlist.setFocus(true)
                else if m.videoGroup.visible
                    m.actionButtonGroup.setFocus(true)
                end if
            end if
        else if (key = "up")
            if (m.videoLabelList.isInFocusChain())
                if (NOT m.top.seriesFavoriteMode) m.videoLabelList.jumpToItem = m.index
                return false
            end if
        end if
    end if

    return true
end function