sub init()
    m.liveTVButton = m.top.findNode("liveTVButton")
    m.moviesButton = m.top.findNode("moviesButton")
    m.seriesButton = m.top.findNode("seriesButton")
    m.loadingIndicator = m.top.findNode("loadingIndicator")
    m.expiredLabel = m.top.findNode("expiredLabel")
    m.categoryList = m.top.findNode("categoryList")
    m.mainGroup = m.top.findNode("mainGroup")
    m.categoryControl = m.top.findNode("categoryControl")
    m.searchButton = m.top.findNode("searchButton")
    m.searchChannelButton = m.top.findNode("searchChannelButton")
    m.searchKeyboard = m.top.findNode("searchKeyboard")
    m.favoriteButton = m.top.findNode("favoriteButton")
    m.liveUpdateLabel = m.top.findNode("liveUpdateLabel")
    m.movieUpdateLabel = m.top.findNode("movieUpdateLabel")
    m.seriesUpdateLabel = m.top.findNode("seriesUpdateLabel")
    m.updateTimer = m.top.findNode("updateTimer")
    m.global.observeField("portalFavourites", "onPortalFavouritesChanged")
    m.updateTimer.observeField("fire", "onUpdateTimerFired")
    m.categoryList.observeField("itemSelected", "onCategorySelected")
    m.liveTVButton.observeField("selected", "showCategories")
    m.moviesButton.observeField("selected", "showCategories")
    m.seriesButton.observeField("selected", "showCategories")
    m.searchButton.observeField("buttonSelected", "showSearchKeyboard")
    m.searchChannelButton.observeField("buttonSelected", "showSearchKeyboard")
    m.favoriteButton.observeField("buttonSelected", "showFavorite")
    m.portalFavouriteCount = 0
    m.focusArr = [m.liveTVButton, m.moviesButton, m.seriesButton]
    m.focusIndex = 0

    m.top.observeField("focusedChild", "onFocusedChildChanged")
    m.top.findNode("background").color = m.global.appColor
    m.top.findNode("companyLogo").uri = m.global.BASE_URL + "/assets/sageiptv.png"
    m.requestBodyList = {
        liveTVButton: {
            action: "get_live_categories"
            allItemAction: "get_live_streams"
            nextAction: "get_live_streams&category_id="
            portalType: "live"
        },
        moviesButton: {
            action: "get_vod_categories"
            allItemAction: "get_vod_streams"
            nextAction: "get_vod_streams&category_id="
            portalType: "vods"
        },
        seriesButton: {
            action: "get_series_categories"
            allItemAction: "get_series"
            nextAction: "get_series&category_id="
            portalType: "series"
        }
    }
    m.portalFavouriteCount = {
        liveTVButton: 0,
        moviesButton: 0,
        seriesButton: 0
    }


    m.contentList = CreateObject("roAssociativeArray")
    m.contentGlobalSearchList = {
        liveTVButton: [],
        moviesButton: [],
        seriesButton: []
    }

    m.allCategoryNode = {}

    m.searchTextBox = m.searchKeyboard.textEditBox
    m.searchTextBox.observeField("text", "onSearchInputChange")
    m.MaxCountDownUpdated = 3600
    m.countDownUpdated = m.MaxCountDownUpdated
    m.initContent = false
    m.msgDialog = m.top.findNode("backDialog")
end sub

function onSearchInputChange(event)
    searchTerm = event.getData()

    if(Len(searchTerm) >= 3)
        if (m.searchMode = "searchButton")
            newContent = createObject("roSGNode", "ContentNode")
            contentList = m.contentList[m.currentShowCategory].getChildren(-1, 0)
    
            for each content in contentList
                if(Instr(0, LCase(content.title), LCase(searchTerm)) <> 0)
                    newContent.appendChild(content.clone(true))
                end if
            end for
            m.searchButton.text = searchTerm
            m.categoryList.content = newContent
        else
            newContent = createObject("roSGNode", "ContentNode")
            contentArr = m.contentGlobalSearchList[m.currentShowCategory]
            for each content in contentArr
                if(Instr(0, LCase(content.title), LCase(searchTerm)) <> 0)
                    newContent.appendChild(content)
                end if
            end for
            m.searchChannelButton.text = searchTerm
            m.categoryControl.seriesFavoriteMode = false
            m.categoryControl.autoPlay = false
            m.categoryControl.content = newContent
            m.categoryControl.visible = true
        end if
    else if (searchTerm = "")
        if (m.searchMode = "searchButton")
            m.categoryList.content = m.contentList[m.currentShowCategory]
            m.searchButton.text = "Search Categories"
        else
            if (m.currentShowCategory = "seriesButton")
                m.searchChannelButton.text = "Search Series"
            else
                m.searchChannelButton.text = "Search Channels"
            end if
        end if
    end if
end function

function onUpdateTimerFired()
    if (m.countDownUpdated > 0)
        updatedSeconds = m.MaxCountDownUpdated - m.countDownUpdated
        updatedStr = updatedSeconds.toStr() + " secs ago"
        if (updatedSeconds > 60)
            seconds = updatedSeconds MOD 60
            minutes = int((updatedSeconds - seconds) / 60)
            
            updatedStr = minutes.toStr() + "m" + seconds.toStr() + "s ago"
        end if
    
        m.liveUpdateLabel.text = "Last updated: " + updatedStr
        m.seriesUpdateLabel.text = "Last updated: " + updatedStr
        m.movieUpdateLabel.text = "Last updated: " + updatedStr
        m.countDownUpdated -= 5
    else
        m.countDownUpdated = m.MaxCountDownUpdated
        getDataForAllCategory()
    end if
end function

function getDataForAllCategory()
    m.streamParams = {
        host: m.defaultPortal.host
        port: m.defaultPortal.port
        username: m.defaultPortal.username
        password: m.defaultPortal.password
    }

    for each nodeId in m.requestBodyList
        m.streamParams.action = m.requestBodyList[nodeId].action
        doRequest("getPortalCategory", sub(response, params)
            if (response.content <> invalid)
                m.contentList[params.nodeId] = response.content
                for i = 0 to response.content.getChildCount() - 1
                    categoryNode = response.content.getChild(i)
                    m.allCategoryNode[categoryNode.id] = categoryNode
                end for
            end if
            
            countCategories(params.nodeId)
            if (m.top.hasfocus()) updateFocusNode()
        end sub, m.streamParams, invalid, { "nodeId": nodeId })
    end for
end function

sub countCategories(nodeId)
    streamParams = {
        host: m.defaultPortal.host
        port: m.defaultPortal.port
        username: m.defaultPortal.username
        password: m.defaultPortal.password
        action: m.requestBodyList[nodeId].allItemAction
    }
    
    m.requiredList[nodeId] = true
    
    doRequest("countPortalCategory", sub(response, params)
        if (response.content <> invalid)
            if (response.content.liveTVButton <> invalid) m.contentGlobalSearchList.liveTVButton = response.content.liveTVButton
            if (response.content.moviesButton <> invalid) m.contentGlobalSearchList.moviesButton = response.content.moviesButton
            if (response.content.seriesButton <> invalid) m.contentGlobalSearchList.seriesButton = response.content.seriesButton
            
            for each categoryId in response.content
                categoryNode = m.allCategoryNode[categoryId]
                if (categoryNode <> invalid AND categoryNode.categoryName <> invalid AND response.content.doesExist(categoryId))
                    categoryNode.title = categoryNode.categoryName + Substitute(" ({0})", response.content[categoryId].toStr())
                end if
            end for
        end if
        if (m.requiredList["done"] <> true)
            check = true
            for each requiredId in m.requiredList
                if ((requiredId <> "done") AND (m.requiredList[requiredId] = false)) check = false
            end for

            if (check)
                m.updateTimer.control = "start"
                onUpdateTimerFired()
                m.loadingIndicator.visible = false
                m.requiredList["done"] = true
            end if
        end if
    end sub, streamParams, invalid, { "nodeId": nodeId })
end sub

sub onCategorySelected()
    m.loadingIndicator.visible = true
    m.item = m.categoryList.content.getChild(m.categoryList.itemFocused)
    m.streamParams.append(m.requestBodyList[m.currentShowCategory])
    m.streamParams.action = m.streamParams.nextAction + m.item.id
    doRequest("getPortalCategoryItemLists", sub(response)
        if (response.content <> invalid)
            m.categoryControl.seriesFavoriteMode = false
            m.categoryControl.autoPlay = true
            if (m.searchChannelButton.text = m.searchTextBox.text)
                m.searchTextBox.text = ""
            end if
            m.categoryControl.content = response.content
            m.categoryControl.visible = true
            m.categoryControl.categoryname = m.item.categoryname
            m.categoryControl.setFocus(true)
        end if
        m.loadingIndicator.visible = false
    end sub, m.streamParams)
end sub

sub showCategories(event) as void
    if (m.loadingIndicator.visible) return
    nodeId = event.getNode()

    m.currentShowCategory = nodeId
    if (m.currentShowCategory = "seriesButton")
        m.searchChannelButton.text = "Search Series"
    else
        m.searchChannelButton.text = "Search Channels"
    end if
    m.categoryList.content = m.contentList[nodeId]
    m.categoryList.visible = true
    m.searchButton.visible = true
    m.searchChannelButton.visible = true
    m.favoriteButton.visible = true
    m.categoryList.setFocus(true)
    m.mainGroup.visible = false
    m.favoriteButton.text = "Favourites (" + m.portalFavouriteCount[m.currentShowCategory].toStr() + ")"
end sub

function onPortalFavouritesChanged()
    m.portalFavouriteCount = {
        liveTVButton: 0,
        moviesButton: 0,
        seriesButton: 0
    }
    for each item in m.portalFavouriteCount
        favouriteAA = m.global.portalFavourites[m.requestBodyList[item].portalType]
        for each streamId in favouriteAA
            if (favouriteAA[streamId] <> invalid) m.portalFavouriteCount[item] += 1
        end for
    end for
    if (m.currentShowCategory <> invalid)
        m.favoriteButton.text = "Favourites (" + m.portalFavouriteCount[m.currentShowCategory].toStr() + ")"
    end if
end function

function showFavorite()
    doRequest("getPortalFavourites", sub(response)
        updatePortalFavouritesGlobal(response.lists)
        portalFavourites =  m.global.portalFavourites
        favoriteContent = createObject("roSGNode", "ContentNode")
        favoriteContent.update({ "isFavoriteNode": true }, true)
        portalType = m.requestBodyList[m.currentShowCategory].portalType

        if (portalFavourites[portalType] <> invalid)
            for each streamId in portalFavourites[portalType]
                streamData = portalFavourites[portalType][streamId]
                if (streamData <> invalid)
                    streamNode = favoriteContent.createChild("ContentNode")
                    streamNode.update(streamData, true)
                end if
            end for
        end if

        if (favoriteContent.getChildCount() = 0)
            showDialog("Error", "No video in this category", m.favoriteButton)
        else
            m.categoryControl.seriesFavoriteMode = (m.currentShowCategory = "seriesButton")
            if (m.searchChannelButton.text = m.searchTextBox.text)
                m.searchTextBox.text = ""
            end if
            m.categoryControl.content = favoriteContent
            m.categoryControl.visible = true
            m.categoryControl.setFocus(true)
        end if

    end sub, {
        baseURL: m.global.BASE_URL,
        token: m.global.token,
    })
end function

sub showDialog(title = "Error", message = "No video in this category" as string, lastFocusNode = invalid)
    if (lastFocusNode <> invalid)
        m.lastFocusNode = lastFocusNode
    else
        m.lastFocusNode = m.top
    end if

    m.msgDialog.buttons = ["OK"]
    m.msgDialog.buttonGroup.focusBitmapUri = "pkg:/images/button_on.9.png"
    m.msgDialog.ObserveField("buttonSelected", "onDialogDismiss")
    m.msgDialog.ObserveField("isBackPressed", "onDialogDismiss")
    m.msgDialog.title = title
    m.msgDialog.message = message
    m.msgDialog.visible = true
    m.msgDialog.setFocus(true)
end sub

sub onDialogDismiss()
    m.msgDialog.visible = false
    m.lastFocusNode.setFocus(true)
end sub

function initContent()
    m.requiredList = {
        liveTVButton: false
        moviesButton: false
        seriesButton: false
    }
    m.countDownUpdated = m.MaxCountDownUpdated

    m.initContent = true
    m.loadingIndicator.visible = true
    mode = getSavedMode()
    modeArr = mode.split("-")
    m.portalId = ""
    if (modeArr.count() = 2)
        m.portalId = modeArr[1]
    end if
    m.defaultPortal = invalid
    doRequest("getPortalLists", sub(response)
        content = response.content
        if (content <> invalid)
            for i = 0 to content.getChildCount() - 1
                item = content.getChild(i)
                if (item.id = m.portalId)
                    m.defaultPortal = item
                    exit for
                end if
            end for
            if (m.defaultPortal = invalid)
                item = content.getChild(0)
                m.portalId = item.id
                m.defaultPortal = item
            end if

            m.global.update({ "defaultPortal": m.defaultPortal }, true)
            if m.top.hasFocus() updateFocusNode()
            getPortalInfo()
            getPortalFavouriteInfo()
            getDataForAllCategory()
        else
            m.loadingIndicator.visible = false
        end if
    end sub, { baseURL: m.global.BASE_URL, email: getEmail() })
end function

sub onFocusedChildChanged()
    if m.top.hasFocus()
        if (m.initContent = false)
            initContent()
        else
            updateFocusNode()
        end if
    end if
end sub

function updatePortalFavouritesGlobal(lists)
    defaultPortalId = m.global.defaultPortal.id
    if (lists.doesExist(defaultPortalId))
        portalFavourites = {}
        portalTypeArr = ["live", "vods", "series"]
        for each portalType in portalTypeArr
            portalFavourites[portalType] = {}
            
            if (lists[defaultPortalId][portalType] <> invalid)
                for each stream in lists[defaultPortalId][portalType]
                    streamData = {
                        "id": stream.stream_id
                        "stream_type": portalType
                        "title": stream.name
                        "stream_id": stream.stream_id
                        "logo": stream.logo
                    }
                    if (streamData.title = invalid) streamData.title = "Stream ID " + stream.stream_id
                    if (streamData.logo = invalid) streamData.logo = ""
                    if (portalType = "series")
                        streamData.series_id = stream.series_id
                        if (streamData.series_id = invalid) streamData.series_id = stream.stream_id.split("-")[0]
                    end if
                    portalFavourites[portalType][stream.stream_id] = streamData
                end for
            end if
            
        end for
        m.global.portalFavourites = portalFavourites
    end if
end function

function getPortalFavouriteInfo()
    doRequest("getPortalFavourites", sub(response)
        updatePortalFavouritesGlobal(response.lists)
    end sub, {
        baseURL: m.global.BASE_URL,
        token: m.global.token,
    })
end function



sub getPortalInfo()
    params = {
        host: m.defaultPortal.host
        port: m.defaultPortal.port
        username: m.defaultPortal.username
        password: m.defaultPortal.password
    }

    doRequest("getPortalInfo", sub(response)
        dateTime = CreateObject("roDateTime")
        if (response.content = invalid OR response.content= "INVALID CONTENT" )
            if response.content = "IPTV SUBSCRIPTION EXPIRED" 
                m.expiredLabel.text = "Your IPTV portal has expired, please check with your IPTV provider"
            else if response.content = "INVALID PORTAL CREDENTIALS"
                m.expiredLabel.text = "Invalid portal credentials, please update your portal info"
            else 
                m.expiredLabel.text = "Invalid response from the portal, please update your portal info"
            end if
        else 
            dateTime.FromSeconds(response.content.toInt())
            m.expiredLabel.text = "Expiration: " + dateTime.AsDateString("no-weekday")
        end if
    end sub, params)
end sub

sub updateFocusNode()
    if (m.focusIndex < 0) m.focusIndex = 0
    if (m.focusIndex > m.focusArr.count() - 1) m.focusIndex = m.focusArr.count() - 1
    m.focusArr[m.focusIndex].setFocus(true)
end sub

function showSearchKeyboard(event)
    nodeId = event.getNode()
    if (m.searchMode <> nodeId)  m.searchTextBox.text = ""
    m.searchMode = nodeId
    if (m.searchMode = "searchButton")
        m.searchKeyboard.setFocus(true)
        m.searchKeyboard.translation = [360, 150]
        m.searchKeyboard.visible = true
        m.categoryControl.visible = false
    else
        m.searchKeyboard.setFocus(true)
        m.searchKeyboard.translation = [700, 100]
        m.searchKeyboard.visible = true
        m.categoryControl.callFunc("ResetControl")
        m.categoryControl.visible = true
    end if

end function

function onKeyEvent(key as string, isPressed as boolean) as boolean
    if isPressed
        ?"[PortalView] onKeyEvent = ";key
        if key = "back"
            if (m.categoryControl.isInFocusChain())
                m.categoryControl.visible = false
                if (m.searchChannelButton.text = m.searchTextBox.text)
                    m.searchTextBox.text = ""
                end if
                m.categoryList.setFocus(true)
                return true
            else if (m.categoryList.isInFocusChain() OR m.searchChannelButton.hasfocus() OR  m.favoriteButton.hasfocus() OR  m.searchKeyboard.hasfocus())
                m.categoryList.visible = false
                m.categoryControl.visible = false
                m.searchButton.visible = false
                m.searchChannelButton.visible = false
                m.favoriteButton.visible = false
                m.searchKeyboard.visible = false
                m.mainGroup.visible = true
                m.searchTextBox.text = ""
                updateFocusNode()
                return true
            else if m.searchKeyboard.visible = true
                m.searchKeyboard.visible = false
                if m.searchMode <> "searchChannelButton"
                    m.searchButton.setFocus(true)
                else
                    m.searchChannelButton.setFocus(true)
                end if
                return true
            end if
        else if key = "up" and (NOT m.mainGroup.visible)
            if m.categoryList.hasfocus() = true
                m.favoriteButton.setFocus(true)
            else if m.favoriteButton.hasfocus() = true
                m.searchButton.setFocus(true)
            else if (m.categoryControl.isInFocusChain())
                m.searchChannelButton.setFocus(true)
            end if
            return true
        else if key = "down"
            if m.searchChannelButton.hasfocus() = true AND m.categoryControl.visible
                m.categoryControl.setFocus(true)
            else if m.searchButton.hasfocus() = true
                m.favoriteButton.setFocus(true)
            else if m.favoriteButton.hasfocus() = true
                m.categoryList.setFocus(true)
            end if
            return true
        else if key = "left"
            if m.categoryList.visible <> true
                m.focusIndex--
                updateFocusNode()
            else if m.searchKeyboard.visible = true
                if (m.searchMode <> "searchChannelButton") 
                    m.categoryList.setFocus(true)
                else
                    if (m.categoryControl.visible)
                        m.categoryControl.setFocus(true)
                    else
                        m.searchChannelButton.setFocus(true)
                    end if
                end if
                m.searchKeyboard.visible = false
            else if (m.categoryControl.isInFocusChain())
                m.categoryList.setFocus(true)
            else if (m.searchChannelButton.hasfocus() = true)
                m.categoryList.setFocus(true)
            end if
            return true
        else if key = "right"
            if (m.searchKeyboard.visible = false)
                if m.categoryList.visible <> true
                    m.focusIndex++
                    updateFocusNode()
                else if (m.categoryList.visible)
                    if (m.categoryControl.visible = false)
                        m.searchChannelButton.setFocus(true)
                    else
                        m.categoryControl.setFocus(true)
                    end if
                end if
            end if
            return true
        end if
    end if

    return false
end function
