sub init()
    m.video = m.top.findNode("selectedVideo")
    m.video.ObserveField("state", "controlvideoplay")
    m.videoLoadingIndicator = m.top.findNode("videoLoadingIndicator")
    m.loadingIndicator = m.top.findNode("loadingIndicator")
    m.actionButtonGroup = m.top.findNode("actionButtonGroup")
    m.top.observeField("focusedChild", "onFocusedChildChanged")
    m.top.findNode("background").color = m.global.appColor
    m.top.findNode("companyLogo").uri = m.global.BASE_URL + "/assets/sageiptv.png"
    m.msgDialog = m.top.findNode("backDialog")
    m.labelListLoadingIndicator = m.top.findNode("labelListLoadingIndicator")
    m.videoGroup = m.top.findNode("videoGroup")
    m.rowList = m.top.findNode("rowList")
    m.rowList.observeField("rowItemSelected", "onRowListItemSelected")
    m.actionButtonGroup.observeField("buttonSelected", "onActionButtonSelected")
    ToggleScreen("small", false)
end sub

function onRowListItemSelected()
    playvideo()
end function

function onActionButtonSelected(event)
    selectedIndex = event.getData()
    actionType = m.actionButtonGroup.getChild(selectedIndex).text
    if (actionType = "Play")
        ToggleScreen("full")
    else if (actionType = "Add to Favourites")
        m.loadingIndicator.visible = true
        favoriteAction("add")
    else if (actionType = "Remove from Favourites")
        m.loadingIndicator.visible = true
        favoriteAction("remove")
    end if
end function

function favoriteAction(action as string)
    if (m.currentStream = invalid) m.currentStream = m.videoLabelList.content.getChild(0)
        m.action = action
        if (action = "add") m.action = action + "&series_id=" + m.seasonInfo.seriesId
        doRequest("portalFavourites", sub(response)
            if (response.status = "success")
                portalFavourites = m.global.portalFavourites
                if (m.actionButtonGroup.getChild(0).text = "Add to Favourites")
                   portalFavourites["series"][m.streamId] = response.status
                    m.actionButtonGroup.buttons = ["Remove from Favourites"]
                else if (m.actionButtonGroup.getChild(0).text = "Remove from Favourites")
                    m.actionButtonGroup.buttons = ["Add to Favourites"]
                    portalFavourites["series"][m.streamId] = invalid
                end if
    
                m.global.portalFavourites = portalFavourites
                m.loadingIndicator.visible = false
                m.actionButtonGroup.setFocus(true)
            end if
        end sub, {
            baseURL: m.global.BASE_URL,
            token: m.global.token,
            portalID: m.global.defaultPortal.id,
            portalType: "series",
            streamID: m.streamId,
            action: m.action,
            streamName: m.seasonInfo.title.Escape()
            steamLogo: m.seasonInfo.cover.Escape()
        })
end function

sub showFavoriteErrorDialog(message as String)
    m.lastFocusNode = m.actionButtonGroup
    m.msgDialog.buttons = ["OK"]
    m.msgDialog.buttonGroup.focusBitmapUri = "pkg:/images/button_on.9.png"
    m.msgDialog.ObserveField("buttonSelected", "onErrorDialogDismiss")
    m.msgDialog.ObserveField("isBackPressed", "onErrorDialogDismiss")
    m.msgDialog.title = "Error"
    m.msgDialog.message = message
    m.msgDialog.visible = true
    m.msgDialog.setFocus(true)
end sub

sub controlvideoplay(event as Object)
    state = event.getData()
    if state = "buffering"
        m.video.enableUI = false
        m.videoLoadingIndicator.visible = true
    else if state = "playing"
        if (m.mode = "full")
            m.video.enableUI = true
        end if    
        m.videoLoadingIndicator.visible = false
    else if (state = "finished" AND m.mode = "full")
        m.videoLabelList.setFocus(true)
        ToggleScreen("small", false)
    else if state = "error"
    end if
end sub

sub playvideo(playingIndex = -1 as integer) as void
    if (NOT m.global.isSubscribe) 
        return
    end if
    focusNode = false
    if playingIndex = -1
        m.index = m.rowList.rowItemFocused
    else
        m.index = [0, 0]
    end if

    if (m.rowList.content = invalid) return
    m.currentStream = m.rowList.content.getChild(m.index[0]).getChild(m.index[1])

    defaultPortal = m.global.defaultPortal
    baseVideoUrl = Substitute("{0}/series/{1}/{2}", defaultPortal.host + ":" + defaultPortal.port, defaultPortal.username, defaultPortal.password)
    sufix = "." + m.currentStream.container_extension
    m.currentStream.update({
        title: m.currentStream.name
        url: Substitute("{0}/{1}{2}", baseVideoUrl,  m.currentStream.id, sufix)
    }, true)

    if (m.global.selectedVideoId <> m.currentStream.id)
        ?"playcontent: ", m.currentStream.url
        m.currentStream.streamFormat = "auto"
        m.video.control = "stop"
        m.video.content = m.currentStream
        m.video.globalCaptionMode = "On"
        m.video.control = "play"
        m.video.visible = true
        ToggleScreen("small", focusNode)
    else
        ToggleScreen("full")
    end if
    m.global.selectedVideoId = m.currentStream.id
end sub

function onSeasonInfoChanged()
    m.seasonInfo = m.top.seasonInfo
    m.streamId = m.seasonInfo.seriesId + "-" + m.seasonInfo.seasonNumber
    if (m.global.portalFavourites["series"][m.streamId] <> invalid)
        m.actionButtonGroup.buttons = ["Remove from Favourites"]
    else
        m.actionButtonGroup.buttons = ["Add to Favourites"]
    end if
end function

function ToggleScreen(mode, focusNode=true)
    m.mode = mode
    if m.mode = "small" then
        m.video.scale = [0.36, 0.36]
        m.video.translation = [460, 106]
        m.video.enableUI = false
        m.videoLoadingIndicator.width = 547
        m.videoLoadingIndicator.height = 309
        m.videoLoadingIndicator.translation= [417, 90]
        m.video.inheritParentTransform = true
        m.videoLoadingIndicator.inheritParentTransform = true
        m.actionButtonGroup.visible = true
        m.rowList.visible = true
        if (focusNode) m.actionButtonGroup.setFocus(true)
    else
        m.videoLoadingIndicator.width = 1280
        m.videoLoadingIndicator.height = 720
        m.videoLoadingIndicator.translation= [0,0]
        m.video.scale = [1, 1]
        m.video.translation = [0, 0]
        if (focusNode) m.video.setFocus(true)
        m.video.enableUI = true
        m.rowList.visible = false
        m.video.inheritParentTransform = false
        m.videoLoadingIndicator.inheritParentTransform = false
        m.actionButtonGroup.visible = false
    end if
end function

sub onFocusedChildChanged()
    if (m.top.hasFocus())
        m.rowList.setFocus(true)
        m.global.selectedVideoId = ""
        playvideo(0)
    end if
end sub

function onContentChanged() as void
    content = m.top.content
    if (content = invalid) return
    m.rowList.content = content
end function

function onKeyEvent(key as string, isPressed as boolean) as boolean
    if isPressed
        if key = "back"
            if (m.rowList.isInFocusChain() = true)
                m.video.control = "stop"
                return false
            else if (m.mode <> "small")
                ToggleScreen("small", false)
                m.rowList.setFocus(true)
            else
                ToggleScreen("small", false)
                m.video.control = "stop"
                return false
            end if
        else if (key = "left")
            if (m.actionButtonGroup.isInFocusChain())
                m.actionButtonGroup.focusButton = 0
                m.rowList.setFocus(true)
            end if
        else if (key = "right")
            if m.rowList.isInFocusChain()
                m.rowList.jumpToRowItem = m.index
                m.actionButtonGroup.setFocus(true)
            end if
        end if
    end if

    return true
end function